/*
 * Copyright(c) 2013 Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Contact Information:
 * Intel Corporation
 */

#include <quark/asset.h>
#include <grub/types.h>

/* Keep track of error triggering condition.  */
struct __quark_error_trigger_struct {
  int no_kernel;
  int no_ramdisk;
  int no_kernel_sig;
  int no_ramdisk_sig;
  int kernel_sig_fail;
  int ramdisk_sig_fail;
  int bad_imr;
  /* How many signatures have been verified so far.  */
  int state_sign_verify;
};

extern struct __quark_error_trigger_struct __quark_err;

void __quark_test_setup (char *);
void __quark_test_imr (void);

#ifdef INTEL_QUARK_TEST

/*
   Failure in fetching an asset results in function returning 0 and errnum
   being set.
 */
#define INTEL_QUARK_ERR_NO_ASSET							\
	(! (errnum = ERR_FILE_NOT_FOUND))

#define __quark_test_asset(x)							\
	if (GRUB_QUARK_ASSET_KERNEL == x && __quark_err.no_kernel)			\
		return INTEL_QUARK_ERR_NO_ASSET;					\
	if (GRUB_QUARK_ASSET_INITRD == x && __quark_err.no_ramdisk)			\
		return INTEL_QUARK_ERR_NO_ASSET;					\
	if (GRUB_QUARK_ASSET_KERNEL_CSBH == x && __quark_err.no_kernel_sig)		\
		return INTEL_QUARK_ERR_NO_ASSET;					\
	if (GRUB_QUARK_ASSET_INITRD_CSBH == x && __quark_err.no_ramdisk_sig)	\
		return INTEL_QUARK_ERR_NO_ASSET;

/*
   Upon failure in verifying signature, IAROM callback only returns 0.
   Possible values for __quark_err.state_sign_verify:
   - 0: we are currently verifying grub.conf
   - 1: we are currently verifying kernel
   - 2: we are currently verifying ramdisk
 */
#define __quark_test_signature(x)							\
	switch (__quark_err.state_sign_verify ++) {				\
	case 0:	/* grub.conf - this is out of scope  */				\
		break;								\
	case 1: /* Kernel  */							\
		if (__quark_err.kernel_sig_fail)					\
			return 0;						\
		break;								\
	case 2: /* Ramdisk  */							\
		if (__quark_err.ramdisk_sig_fail)					\
			return 0;						\
		break;								\
	default:								\
		break;								\
	}

#else /* !INTEL_QUARK_TEST  */

#define __quark_test_asset(x)							\
	do {} while(0);
#define __quark_test_signature(x)							\
	do {} while(0);

#endif /* INTEL_QUARK_TEST  */

